/*!
 * vue-editor-js v1.0.1 
 * (c) 2019 ChangJoo Park<pcjpcj2@gmail.com>
 * Released under the MIT License.
 */
import VueCompositionApi, { createComponent, reactive, onMounted, watch } from '@vue/composition-api';
import EditorJS from '@editorjs/editorjs';

function _typeof(obj) {
  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(source, true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(source).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var DEFAULT_OBJECT_PROP = {
  type: Object,
  default: function _default() {},
  required: false
};
var DEFAULT_BOOLEAN_PROP = {
  type: Boolean,
  default: function _default() {
    return false;
  },
  required: false
};
var PLUGIN_PROPS_TYPE = {
  type: [Boolean, Object],
  default: function _default() {
    return false;
  },
  required: false
};
var PLUGIN_PROPS = ['header', 'list', 'code', 'inlineCode', 'embed', 'link', 'marker', 'table', 'raw', 'delimiter', 'quote', 'image', 'warning', 'paragraph', 'checklist'];
var PLUGINS = {
  header: require('@editorjs/header'),
  list: require('@editorjs/list'),
  image: require('@editorjs/image'),
  inlineCode: require('@editorjs/inline-code'),
  embed: require('@editorjs/embed'),
  quote: require('@editorjs/quote'),
  marker: require('@editorjs/marker'),
  code: require('@editorjs/code'),
  link: require('@editorjs/link'),
  delimiter: require('@editorjs/delimiter'),
  raw: require('@editorjs/raw'),
  table: require('@editorjs/table'),
  warning: require('@editorjs/warning'),
  paragraph: require('@editorjs/paragraph'),
  checklist: require('@editorjs/checklist')
};
function useTools(props) {
  var pluginKeys = Object.keys(PLUGINS);

  var tools = _objectSpread2({}, props.customTools);
  /**
   * When plugin props are empty, enable all plugins
   */


  if (pluginKeys.every(function (p) {
    return !props[p];
  })) {
    pluginKeys.forEach(function (key) {
      return tools[key] = {
        class: PLUGINS[key]
      };
    });
    return tools;
  }

  pluginKeys.forEach(function (key) {
    var prop = props[key];

    if (!prop) {
      return;
    }

    tools[key] = {
      class: PLUGINS[key]
    };

    if (_typeof(prop) === 'object') {
      var options = Object.assign({}, props[key]);
      delete options['class']; // Prevent merge wrong `class`

      tools[key] = Object.assign(tools[key], options);
    }
  });
  return tools;
}

var script = createComponent({
  name: 'vue-editor-js',
  props: _objectSpread2({
    holderId: {
      type: String,
      default: function _default() {
        return 'codex-editor';
      },
      required: false
    },
    autofocus: DEFAULT_BOOLEAN_PROP,
    initData: DEFAULT_OBJECT_PROP,
    customTools: DEFAULT_OBJECT_PROP
  }, PLUGIN_PROPS.map(function (_) {
    return PLUGIN_PROPS_TYPE;
  })),
  setup: function setup(props, context) {
    var state = reactive({
      editor: null
    });

    function initEditor(props) {
      destroyEditor();
      var holder = props.holderId,
          autofocus = props.autofocus,
          data = props.initData;
      state.editor = new EditorJS({
        holder: holder,
        autofocus: autofocus,
        data: data,
        tools: useTools(props),
        onReady: function onReady() {
          return context.emit('ready');
        },
        onChange: function onChange() {
          return context.emit('change');
        }
      });
    }

    function destroyEditor() {
      if (state.editor) {
        state.editor.destroy();
        state.editor = null;
      }
    }

    function save() {
      if (!state.editor) {
        return;
      }

      state.editor.save().then(function (result) {
        return context.emit('save', result);
      }).catch(function (err) {
        return console.error(err);
      });
    }

    onMounted(function (_) {
      return initEditor(props);
    });
    watch(function (_) {
      return props.initData;
    }, function (_) {
      return initEditor;
    });
    return {
      props: props,
      state: state,
      save: save
    };
  }
});

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier
/* server only */
, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
  if (typeof shadowMode !== 'boolean') {
    createInjectorSSR = createInjector;
    createInjector = shadowMode;
    shadowMode = false;
  } // Vue.extend constructor export interop.


  var options = typeof script === 'function' ? script.options : script; // render functions

  if (template && template.render) {
    options.render = template.render;
    options.staticRenderFns = template.staticRenderFns;
    options._compiled = true; // functional template

    if (isFunctionalTemplate) {
      options.functional = true;
    }
  } // scopedId


  if (scopeId) {
    options._scopeId = scopeId;
  }

  var hook;

  if (moduleIdentifier) {
    // server build
    hook = function hook(context) {
      // 2.3 injection
      context = context || // cached call
      this.$vnode && this.$vnode.ssrContext || // stateful
      this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext; // functional
      // 2.2 with runInNewContext: true

      if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
        context = __VUE_SSR_CONTEXT__;
      } // inject component styles


      if (style) {
        style.call(this, createInjectorSSR(context));
      } // register component module identifier for async chunk inference


      if (context && context._registeredComponents) {
        context._registeredComponents.add(moduleIdentifier);
      }
    }; // used by ssr in case component is cached and beforeCreate
    // never gets called


    options._ssrRegister = hook;
  } else if (style) {
    hook = shadowMode ? function () {
      style.call(this, createInjectorShadow(this.$root.$options.shadowRoot));
    } : function (context) {
      style.call(this, createInjector(context));
    };
  }

  if (hook) {
    if (options.functional) {
      // register for functional component in vue file
      var originalRender = options.render;

      options.render = function renderWithStyleInjection(h, context) {
        hook.call(context);
        return originalRender(h, context);
      };
    } else {
      // inject component registration as beforeCreate hook
      var existing = options.beforeCreate;
      options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
    }
  }

  return script;
}

var normalizeComponent_1 = normalizeComponent;

/* script */
const __vue_script__ = script;

/* template */
var __vue_render__ = function () {var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{attrs:{"id":"vue-editor-js"}},[_c('div',{attrs:{"id":_vm.props.holderId}}),_vm._v(" "),_c('button',{staticStyle:{"display":"none"},attrs:{"id":((_vm.props.holderId) + "-button")},on:{"click":_vm.save}})])};
var __vue_staticRenderFns__ = [];

  /* style */
  const __vue_inject_styles__ = undefined;
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  

  
  var EditorComponent = normalizeComponent_1(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    undefined,
    undefined
  );

var version = '1.0.1';
function install(Vue) {
  if (install.installed) return;
  install.installed = true;
  Vue.use(VueCompositionApi);
  Vue.component('Editor', EditorComponent);
}
var plugin = {
  install: install,
  version: version
};
var Editor = EditorComponent;
var GlobalVue = null;

if (typeof window !== 'undefined') {
  GlobalVue = window.Vue;
} else if (typeof global !== 'undefined') {
  GlobalVue = global.Vue;
}

if (GlobalVue) {
  GlobalVue.use(plugin);
}

export default plugin;
export { Editor, install };
